//-----------------------------------------------------------------------------
// name: mo_compass.h
// desc: MoPhO API for compass 
//
// authors: Nick Bryan
//          Jieun Oh
//          Jorge Herrera
//          Ge Wang
//
//    date: Fall 2009
//    version: 0.1
//
// Stanford Mobile Phone Orchestra
//     http://mopho.stanford.edu/
//-----------------------------------------------------------------------------
#ifndef __MO_COMPASS_H__
#define __MO_COMPASS_H__

#include "mo_def.h"
#import <vector>
#import <CoreLocation/CoreLocation.h>


@interface CompassDelegate : NSObject <CLLocationManagerDelegate>
{
	CLLocationManager *locationManager;
}
@property (nonatomic, retain) CLLocationManager *locationManager;

@end


// type definition for accelerometer callback function
typedef void (* MoCompassCallback)( CLHeading * heading, void * data );

//-----------------------------------------------------------------------------
// name: class MoLocation
// desc: location stuff, GPS/Edge/Wifi + Compass (singleton)
//-----------------------------------------------------------------------------
class MoCompass
{
public: // setting values
    void update( CLHeading * heading );
    // set the current compass offset
    void setOffset();
    // clear the current compass offset
    void clearOffset();
    
    double getMagneticOffset();
    double getTrueOffset();

public: // getting values
    // get the magnetic heading of the compass
    double getMagneticHeading() const;

    // get the magnetic heading of the compass
    double getTrueHeading() const;

    // get the estimated accuracy of the compass
    double getAccuracy() const;

    // register a callback to be invoked on subsequent updates
    void add( const MoCompassCallback & callback, void * data );  // TODO: change to addCallback

    // unregister a callback
    void remove( const MoCompassCallback & callback );  // TODO: change to removeCallback

public: // static members
    static MoCompass * instance();
    static MoCompass * ourInstance;

private:
    MoCompass();
    ~MoCompass();

    // current values;
    CLHeading * m_heading;

    double m_magneticHeading, m_trueHeading;
    CLLocationDirection m_accuracy;

    // store an offset 
    double m_trueOffset, m_magneticOffset;

    CompassDelegate * compassDelegate;

    // queue of callbacks
    std::vector< MoCompassCallback > m_clients;
    std::vector<void *> m_clientData;//TODO: update the add and remove functions with the void * data

    // internal setup of the compass
    void setup();
};


#endif
