//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// name: mo_location.h
// desc: MoPhO API for compass 
//
// authors: Nick Bryan
//          Jieun Oh
//          Jorge Herrera
//          Ge Wang
//
//    date: Fall 2009
//    version: 0.1
//
// Stanford Mobile Phone Orchestra
//     http://mopho.stanford.edu/
//-----------------------------------------------------------------------------
#ifndef __MO_LOCATION_H__
#define __MO_LOCATION_H__

#include "mo_def.h"
#import <CoreLocation/CoreLocation.h>
#include <vector>


@interface LocationDelegate : NSObject <CLLocationManagerDelegate>
{
    CLLocationManager *locationManager;
}
@property (nonatomic, retain) CLLocationManager *locationManager;

@end


// type definition for accelerometer callback function
typedef void (* MoLocationCallback)( CLLocation * newLocation, 
                                    CLLocation * oldLocation,
                                    void * data);

//-----------------------------------------------------------------------------
// name: class MoLocation
// desc: location stuff, GPS/Edge/Wifi + Compass (singleton)
//-----------------------------------------------------------------------------
class MoLocation
{
public: // setting values
    void update( CLLocation * newLoc, CLLocation * oldLoc);
    
public: // getting values
    // register a callback to be invoked on subsequent updates
    void add(const MoLocationCallback & callback, void * data );  // TODO: change to addCallback
    
    // unregister a callback
    void remove(const MoLocationCallback & callback );  // TODO: change to removeCallback

    // get the newest location
    CLLocation * getLocation();
    
    // get the previous location
    CLLocation * getOldLocation();

    /*
    Location Attributes
    coordinate  property
    altitude  property
    horizontalAccuracy  property
    verticalAccuracy  property
    timestamp  property
    – description
    Measuring the Distance Between Coordinates
    – getDistanceFrom:
    Getting Speed and Course Information
    speed  property
    course  property
    */

public: // static members
    static MoLocation * instance();
    static MoLocation * ourInstance;
    
private:
    MoLocation();
    ~MoLocation();

    // current values;
    CLLocation * m_newLocation;
    CLLocation * m_oldLocation;

    LocationDelegate * locationDelegate;
    
    // queue of callbacks
    std::vector< MoLocationCallback > m_clients;
    std::vector<void *> m_clientData;//TODO: update the add and remove functions with the void * data

    // set up the Location delegate
    void setup();
};


#endif
